@@ -54,7 +54,7 @@
 //!     return _api.my_game_function(input)
 //! ```
 
-use interoptopus::lang::c::{CType, ConstantValue, EnumType, FnPointerType, Function, PrimitiveValue};
+use interoptopus::lang::c::{CType, CompositeType, ConstantValue, EnumType, FnPointerType, Function, PrimitiveValue};
 use interoptopus::patterns::class::Class;
 use interoptopus::patterns::{LibraryPattern, TypePattern};
 use interoptopus::util::{longest_common_prefix, safe_name};
@@ -184,6 +184,7 @@ pub trait InteropCPythonCFFI: Interop {
         for the_type in self.library().ctypes() {
             match the_type {
                 CType::Enum(e) => self.write_enum(w, e)?,
+                CType::Composite(c) => self.write_struct(w, c)?,
                 CType::Pattern(TypePattern::SuccessEnum(e)) => self.write_enum(w, e.the_enum())?,
                 _ => {}
             }
@@ -192,6 +193,67 @@ pub trait InteropCPythonCFFI: Interop {
         Ok(())
     }
 
+    fn write_struct(&self, w: &mut IndentWriter, e: &CompositeType) -> Result<(), Error> {
+        let docs = e.meta().documentation().lines().join("\n");
+
+        w.indented(|w| writeln!(w, r#"class {}(object):"#, e.rust_name()))?;
+        w.indent();
+        w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
+
+        // Ctor
+        w.indented(|w| writeln!(w, r#"def __init__(self):"#))?;
+        w.indent();
+        w.indented(|w| writeln!(w, r#"global _api, ffi"#))?;
+        w.indented(|w| writeln!(w, r#"self._ctx = ffi.new("{}[]", 1)[0]"#, e.rust_name()))?;
+        w.unindent();
+        w.newline()?;
+
+        // Array constructor
+        w.indented(|w| writeln!(w, r#"def array(n):"#))?;
+        w.indent();
+        w.indented(|w| writeln!(w, r#"global _api, ffi"#))?;
+        w.indented(|w| writeln!(w, r#"return ffi.new("{}[]", n)"#, e.rust_name()))?;
+        w.unindent();
+        w.newline()?;
+
+        for field in e.fields() {
+            let docs = field.documentation().lines().join("\n");
+            w.indented(|w| writeln!(w, r#"@property"#))?;
+            w.indented(|w| writeln!(w, r#"def {}(self):"#, field.name()))?;
+            w.indent();
+            w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
+            w.indented(|w| writeln!(w, r#"return self._ctx.{}"#, field.name()))?;
+            w.unindent();
+            w.newline()?;
+
+            let docs = field.documentation().lines().join("\n");
+            w.indented(|w| writeln!(w, r#"@{}.setter"#, field.name()))?;
+            w.indented(|w| writeln!(w, r#"def {}(self, value):"#, field.name()))?;
+            w.indent();
+            w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
+            // We also write _ptr to hold on to any allocated object created by CFFI. If we do not
+            // then we might assign a pointer in the _ctx line below, but once the parameter (the CFFI handle)
+            // leaves this function the handle might become deallocated and therefore the pointer
+            // becomes invalid
+            w.indented(|w| writeln!(w, r#"self._ptr_{} = value"#, field.name()))?;
+            w.indented(|w| writeln!(w, r#"self._ctx.{} = value"#, field.name()))?;
+            w.unindent();
+            w.newline()?;
+        }
+
+        // General Getter (gives weird errors
+        // w.indented(|w| writeln!(w, r#"def __get__(self, obj, objtype):"#))?;
+        // w.indent();
+        // w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
+        // w.indented(|w| writeln!(w, r#"return getattr(obj, self.ctx)"#))?;
+        // w.unindent();
+        // w.newline()?;
+
+        w.unindent();
+
+        Ok(())
+    }
+
     fn write_enum(&self, w: &mut IndentWriter, e: &EnumType) -> Result<(), Error> {
         let docs = e.meta().documentation().lines().join("\n");
 
@@ -253,6 +315,16 @@ pub trait InteropCPythonCFFI: Interop {
             w.indent();
             w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
             w.indented(|w| writeln!(w, r#"global _api"#))?;
+
+            // Determine if the function was called with a wrapper we produced which as a private `_ctx`.
+            // If so, use that instead. Otherwise, just pass parameters and hope for the best.
+            for param in function.signature().params() {
+                w.indented(|w| writeln!(w, r#"if hasattr({}, "_ctx"):"#, param.name()))?;
+                w.indent();
+                w.indented(|w| writeln!(w, r#"{p} = {p}._ctx"#, p = param.name()))?;
+                w.unindent()
+            }
+
             w.indented(|w| writeln!(w, r#"return _api.{}({})"#, function.name(), &args))?;
             w.unindent();
 
@@ -294,7 +366,7 @@ pub trait InteropCPythonCFFI: Interop {
 
         match function.signature().rval() {
             CType::Pattern(TypePattern::SuccessEnum(e)) => {
-                w.indented(|w| writeln!(w, r#"rval = _api.{}({}, {})"#, function.name(), ctx, &args))?;
+                w.indented(|w| writeln!(w, r#"rval = {}.{}({}, {})"#, self.config().raw_fn_namespace, function.name(), ctx, &args))?;
                 w.indented(|w| writeln!(w, r#"if rval == {}.{}:"#, e.the_enum().rust_name(), e.success_variant().name()))?;
                 w.indent();
                 w.indented(|w| writeln!(w, r#"return None"#))?;
@@ -328,6 +400,12 @@ pub trait InteropCPythonCFFI: Interop {
         w.indented(|w| writeln!(w, r#"def __init__(self, {}):"#, args))?;
         w.indent();
         w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
+        for param in class.constructor().signature().params().iter().skip(1) {
+            w.indented(|w| writeln!(w, r#"if hasattr({}, "_ctx"):"#, param.name()))?;
+            w.indent();
+            w.indented(|w| writeln!(w, r#"{p} = {p}._ctx"#, p = param.name()))?;
+            w.unindent()
+        }
         w.indented(|w| writeln!(w, r#"global _api, ffi"#))?;
         w.indented(|w| writeln!(w, r#"self.ctx = ffi.new("{}**")"#, context_type_name))?;
         self.write_pattern_class_success_enum_aware_rval(w, class, class.constructor(), false)?;
@@ -351,7 +429,7 @@ pub trait InteropCPythonCFFI: Interop {
             w.indented(|w| writeln!(w, r#"def {}(self, {}):"#, function.name().replace(&common_prefix, ""), &args))?;
             w.indent();
             w.indented(|w| writeln!(w, r#""""{}""""#, docs))?;
-            w.indented(|w| writeln!(w, r#"global _api"#))?;
+            w.indented(|w| writeln!(w, r#"global {}"#, self.config().raw_fn_namespace))?;
 
             self.write_pattern_class_success_enum_aware_rval(w, class, function, true)?;
 
@@ -372,6 +450,17 @@ pub trait InteropCPythonCFFI: Interop {
         w.indented(|w| writeln!(w, r#"""""#))?;
         Ok(())
     }
+
+    fn write_utils(&self, w: &mut IndentWriter) -> Result<(), Error> {
+        w.indented(|w| writeln!(w, r#"def ascii_string(x):"#))?;
+        w.indent();
+        w.indented(|w| writeln!(w, r#""""Must be called with a b"my_string".""""#))?;
+        w.indented(|w| writeln!(w, r#"global ffi"#))?;
+        w.indented(|w| writeln!(w, r#"return ffi.new("char[]", x)"#))?;
+        w.unindent();
+        w.newline()?;
+        Ok(())
+    }
 }
 
 impl Interop for Generator {
@@ -407,6 +496,10 @@ impl Interop for Generator {
         w.newline()?;
         w.newline()?;
 
+        self.write_utils(w)?;
+        w.newline()?;
+        w.newline()?;
+
         Ok(())
     }
 }
